from .cwebp import Cwebp as Cwebp
from .gifsicle import Gifsicle as Gifsicle
from .jpegoptim import Jpegoptim as Jpegoptim
from .optipng import Optipng as Optipng
from .pngquant import Pngquant as Pngquant
